@@ -1,25 +1,20 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
+# Created on Wed Apr  1 14:56:15 2020
+# Author: Xiaoding Ming
+
 """
-[file name]
+
+Demo functions
+==============
+
 To do:
-    [Write the tasks to do in this script]
------------------    
-Created on Thu Apr 23 10:08:12 2020
+    * Create the demo
 
-@author: Xiaodong Ming
-"""
+---------------
 
-#!/usr/bin/env python3
-# -*- coding: utf-8 -*-
-"""
-init
-To do:
-    initialize a package
-Created on Wed Apr  1 14:56:15 2020
-
-@author: Xiaodong Ming
 """
+import os
 import pkg_resources
 import numpy as np
 from .InputHipims import InputHipims
@@ -29,20 +24,21 @@ from .Raster import Raster
 def demo_input(num_of_sections=1, set_example_inputs=True,
                figname=None, dpi=200, **kwargs):
     """ A demonstration to generate a hipims input object
-    set_example_inputs: (True|False) if True, initial condition, boundary
-        condition, rainfall source, and gauge postion will be set to the input 
-        object according to sample data.
-    figname: (string) if given, a domain map will saved
+    
+    Args:
+        set_example_inputs: (True|False) if True, initial condition, boundary condition, 
+            rainfall source, and gauge postion will be set to the input object according to sample data.
+        figname: (string) if given, a domain map will saved
+
     """
     dem_file = pkg_resources.resource_filename(__name__,
-                                             'sample/Example_DEM.gz')
+                                             'sample/DEM.gz')
     obj_in = InputHipims(dem_data=dem_file, num_of_sections=num_of_sections)
     if set_example_inputs:
         __set_defaul_input(obj_in)
     # show model summary print(obj_in)
     obj_in.Summary.display()
-    fig, ax = obj_in.domain_show(relocate=True, scale_ratio=1000, 
-                                 cax_str='DEM (m)', **kwargs)
+    fig, ax = obj_in.domain_show(relocate=True, scale_ratio=1000, **kwargs)
     ax.set_title('The Upper Lee catchment')
     if figname is not None:
         fig.savefig(figname, dpi=dpi)
@@ -50,7 +46,13 @@ def demo_input(num_of_sections=1, set_example_inputs=True,
 
 def demo_output(case_folder, num_of_sections=1):
     """ A demonstration to generate a hipims output object
-    a input folder and files must be created before using this function
+
+    Args:
+        case_folder: (string) path to case folder
+        num_of_sections: (int) number of domains
+    
+    Note:
+        a input folder and files must be created before using this function
     """
     obj_out = OutputHipims(case_folder=case_folder,
                            num_of_sections=num_of_sections)
@@ -58,7 +60,9 @@ def demo_output(case_folder, num_of_sections=1):
 
 def demo_raster(figname=None):
     """ A demonstration to read and show raster files
-    figname: the file name to save the figure
+
+    Args:
+        figname: (string) the file name to save the figure
     """
     dem_file = pkg_resources.resource_filename(__name__,
                                              'sample/Example_DEM.asc')
@@ -67,20 +71,23 @@ def demo_raster(figname=None):
     ax.set_title('The Upper Lee catchment DEM (mAOD)')
     return obj_ras
 
-def get_sample_data():
+def get_sample_data(return_path=False):
     """ Get sample data for demonstartion
-    Return:
-        obj_ras: a DEM raster object
-        demo_data: a dictionary with boundary_condition, rain_source, and 
-            gauges_pos data
+
+    Returns:
+        A DEM raster object and a dictionary with boundary_condition, rain_source, and gauges_pos data
     """
     dem_file = pkg_resources.resource_filename(__name__,
-                                             'sample/Example_DEM.gz')
-    obj_ras = Raster(dem_file)
-    demo_data_file = pkg_resources.resource_filename(__name__,
-                                             'sample/Example_data.npy')
-    demo_data = np.load(demo_data_file, allow_pickle='TRUE').item()
-    return obj_ras, demo_data
+                                             'sample/DEM.gz')
+    if return_path:
+        sample_path = os.path.dirname(dem_file)
+        return sample_path
+    else:
+        obj_ras = Raster(dem_file)
+        demo_data_file = pkg_resources.resource_filename(__name__,
+                                                 'sample/Example_data.npy')
+        demo_data = np.load(demo_data_file, allow_pickle='TRUE').item()
+        return obj_ras, demo_data
     
 # =============private functions==================
 def __set_defaul_input(obj_in):
@@ -96,9 +103,9 @@ def __set_defaul_input(obj_in):
     h0[h0 < 50] = 0
     h0[h0 >= 50] = 1
     # set initial water depth (h0) and velocity (hU0x, hU0y)
-    obj_in.set_parameter('h0', h0)
-    obj_in.set_parameter('hU0x', h0*0.0001)
-    obj_in.set_parameter('hU0y', h0*0.0002)
+    obj_in.set_initial_condition('h0', h0)
+    obj_in.set_initial_condition('hU0x', h0*0.0001)
+    obj_in.set_initial_condition('hU0y', h0*0.0002)
     # define boundary condition
     bound_list = demo_data['boundary_condition']
     obj_in.set_boundary_condition(bound_list, outline_boundary='fall')
@@ -107,4 +114,4 @@ def __set_defaul_input(obj_in):
     obj_in.set_rainfall(rain_mask=0, rain_source=rain_source)
     # define and set monitor positions
     gauges_pos = demo_data['gauges_pos']
-    obj_in.set_gauges_position(gauges_pos)
\ No newline at end of file
+    obj_in.set_gauges_position(gauges_pos)
